Program
  Uses pure2d, basilisk, keyset

Const
   MAX_SCRIPT_LENGTH = 20

Type Listeners = Record
   ListenerFunc : Array [MAX_SCRIPT_LENGTH] of String
   ListenerParams : Array [MAX_SCRIPT_LENGTH] of String
   ListenerPriority : Integer
   ListenerCondition : String
   ListenerCompare : Integer
   ListenerLength : Integer
EndType

Procedure Error(msg: String)
Begin
   MessageBox(msg,"Real Time Scripting Example Error")
End

Procedure LoadRealTimeScript(fname: String)
Var
 myfile : Element
 fileIn : String
 
 ladddone : Boolean
 laddcount : Integer
 laddcomp : String
Begin
 myfile = ReadFile(fname)
 
 While Not EOF(myfile)  
   fileIn = TrimSpaces(ReadLn(myfile))
 
   Conditions
       (Left(fileIn,1) = "#"):
       (fileIn=""):
       (Left(fileIn,10)="<listener "):
       
           Listener = NewItem(ListenerList)
           
           Listener.ListenerCondition = AttributeAsString(fileIn,"condition")
           Listener.ListenerPriority  = AttributeAsInteger(fileIn,"priority")
           
           laddcomp = Lower(AttributeAsString(fileIn,"compare"))
           Conditions
               (laddcomp="integer"): Listener.ListenerCompare = VVAR_INTEGER
               (laddcomp="real"): Listener.ListenerCompare = VVAR_REAL
               (laddcomp="string"): Listener.ListenerCompare = VVAR_STRING
               (laddcomp="boolean"): Listener.ListenerCompare = VVAR_BOOLEAN
               Default: Error("Unknown comparason mode: "+laddcomp)
           EndConditions
       
           ladddone = FALSE
           laddcount = 0
           
           While ladddone=FALSE
               If EOF(myfile) then ladddone=TRUE
               fileIn = TrimSpaces(ReadLn(myfile))
               If fileIn = "</listener>" then
                   ladddone = TRUE
               Else
                   If fileIn <> "" then
                       Listener.ListenerFunc[laddcount] = AttributeAsString(fileIn,"name")
                       Listener.ListenerParams[laddcount] = ","+AttributeAsString(fileIn,"params")
                       Inc(laddcount)
                   Endif
               Endif                
           Wend
           
           Listener.ListenerLength = laddcount - 1
           
       Default:
           Error("Error reading script instrucion: "+fileIn)
   EndConditions
 Wend

 CloseFile(myfile)
End

Procedure UpdateListeners()
Var
   count : static Integer
   sdo : Boolean
   i : Integer
   
   lfunc, lparams : String
   sintvalue : Integer
Begin
   Inc(count)
   If (count>10000) then count = 1
   
   Loop Listener Through ListenerList
    If (count Mod Listener.ListenerPriority = 0) then
           sdo = FALSE
           
           Case Listener.ListenerCompare of
               (VVAR_INTEGER): If IntegerExpressionIsTrue(ListenerVarReplace(Listener.ListenerCondition)) then sdo=TRUE
               (VVAR_REAL): If RealExpressionIsTrue(ListenerVarReplace(Listener.ListenerCondition)) then sdo=TRUE
               (VVAR_STRING): If StringExpressionIsTrue(ListenerVarReplace(Listener.ListenerCondition)) then sdo=TRUE
               (VVAR_BOOLEAN): If BooleanExpressionIsTrue(ListenerVarReplace(Listener.ListenerCondition)) then sdo=TRUE
           EndCase

           If sdo Then
               For i = 0 to Listener.ListenerLength
                   lfunc = Listener.ListenerFunc[i]
                   lparams = ListenerVarReplace(Listener.ListenerParams[i])
               
                   Conditions
                       (lfunc="cls"): Cls
                       (lfunc="setint"):
                            sintvalue = ParameterAsInteger(lparams,2,TRUE)
                            FreeVirtualVariable(ParameterAsString(lparams,1))
                            MapVirtualInteger(ParameterAsString(lparams,1),sintvalue)
                       (lfunc="destroy"): Free(listener); Exit
                       (lfunc="text"):
                           Text(ParameterAsInteger(lparams,1,TRUE),ParameterAsInteger(lparams,2,TRUE),NiceText(ParameterAsString(lparams,3,TRUE)))
                       (lfunc="rect"):
                           Rect(ParameterAsInteger(lparams,1,TRUE), ParameterAsInteger(lparams,2,TRUE), ParameterAsInteger(lparams,3,TRUE), ParameterAsInteger(lparams,4,TRUE), ToRGBA(ParameterAsInteger(lparams,5,TRUE),ParameterAsInteger(lparams,6,TRUE),ParameterAsInteger(lparams,7,TRUE)),TRUE)
                       Default: Error("Unknown function: '"+lfunc+"'")
                   EndConditions
               Next
           Endif
           
       Endif
   EndLoop
   
End

Function ListenerVarReplace(str: String) : String
Begin
   str = Replace(str, "MOUSEX", MouseX, FALSE)
   str = Replace(str, "MOUSEY", MouseY, FALSE)
   
   result = str
End

Function NiceText(str: String) : String
Begin
   str = Replace(str, "#COMMA#",",", FALSE)
   
   result = str
End

Var
   ListenerList : List of Listeners
Listener : ^Listeners

Begin
   InitBasilisk(TRUE)

   ReadSettings("settings.bsd")
   SetAppName(VirtualString("win_apptitle"))   
   OpenScreen(VirtualInteger("win_width"),VirtualInteger("win_height"),VirtualInteger("win_depth"),VirtualBoolean("win_mode"),VirtualInteger("win_flags"))
   FreeVirtualVariablesPrefixed("win_")
   
   LoadRealTimeScript("script.brt")
   
   While Not ExitRequested
       UpdateListeners()
       Flip
       Pause(1)
       If (KeyHits(VK_ESCAPE) > 0) then RequestExit
   Wend
   
   VirtualVariableDump("vvars-end.txt")
End